--[[ Azuarc's Level Guide v. 1.0

Azuarc's Level Guide is a simple addon that take's Azuarc's excellent online levelling guide (available at http://wow.allakhazam.com/db/guides.html?guide=963) and presents it in an easy to access format for on-line reference without the need to alt-tab back and forth.
 
The UI is based on Gello's TinyPad, a lightweight notepad mod.  Special thanks to Malcolm Chambers for creating the TinyPadEditor to make entering this data in *much* easier!

Azuarc's level guide is used by permission of Azuarc and Allakhazam.  

NOTE: while the guides are mostly presented in their original form, there are certain places that some comments have been removed for length.  In addition, all of the maps and graphics have been stripped, so first-time users are encouraged to review the original guides before trying to follow the on-line version.
	
]]

-- SavedVariables
AzuarcLGPages = { "" } -- numerically-indexed table of pages
AzuarcLGSettings = {} -- Lock, Font, Chapter, Page
ALGMMBSettings = {} -- hold minimap button settings

AzuarcLG = {

	-- tooltip info for controls
	controls = {
		{"ALGDDMenuButton","Select Chapter","Select which chapter of the guide to view."},
		{"AzuarcLGStart","First Page","Go to first scene"},
		{"AzuarcLGLeft","Flip Back","Go back one scene"},
		{"AzuarcLGRight","Flip Forward","Go forward one scene"},
		{"AzuarcLGEnd","Last Page","Go to the last scene"},
		{"AzuarcLGClose","Close Guide","Close the Azuarc Level Guide"},
		{"AzuarcLGLock","Lock","Lock or Unlock window"},
		{"AzuarcLGFont","Font","Cycle through different fonts"},
		{"AzuarcLGSearch","Options","Search pages for text, change fonts or lock window"},
		{"AzuarcLGSearchNext","Find Next","Find next page with this text"},
		{"AzuarcLGSearchEditBox","Search Criterea","Enter the text to search for here"},
		{"AzuarcLGUndo","Undo","Revert page to last saved text"},
	},

	-- list of fonts to cycle through
	fonts = {
		{"Fonts\\FRIZQT__.TTF",10},
		{"Fonts\\FRIZQT__.TTF",12}, -- default
		{"Fonts\\FRIZQT__.TTF",16},
		{"Fonts\\ARIALN.TTF",12},
		{"Fonts\\ARIALN.TTF",16},
		{"Fonts\\ARIALN.TTF",20},
		{"Fonts\\MORPHEUS.ttf",16,"OUTLINE"},
		{"Fonts\\MORPHEUS.ttf",24,"OUTLINE"}
		-- add fonts here
	},

	current_page = 1, -- page currently viewed
	first_use = 1 -- whether this was used this session
}


BINDING_HEADER_AZUARCLG = "Azuarc's Level Guide"
SLASH_COMMAND_MMB = "MINIMAP"
SLASH_COMMAND_MMB_HELP = "minimap [ON|OFF]"
ALG_MINIMAP_BUTTON = "Azuarc's Leveling Guide"
ALGHUD_ICON = "Interface\\AddOns\\AzuarcLevelGuide\\ALGIcon";

function AzuarcLG.OnLoad()

	-- register events 
	this:RegisterEvent("VARIABLES_LOADED") -- create buttons anytime after VARIABLES_LOADED
	this:RegisterEvent("PLAYER_LOGIN") -- initialization

	-- set up frame
	this:SetMinResize(236,96)
	table.insert(UISpecialFrames,"AzuarcLGFrame")
	table.insert(UISpecialFrames,"AzuarcLGSearchFrame")

      -- set up slash handler
	SlashCmdList["AZUARCLGSLASH"] = AzuarcLG.SlashHandler
	SLASH_AZUARCLGSLASH1 = "/alg"
	SLASH_AZUARCLGSLASH2 = "/azuarc"
	
end

function AzuarcLG.OnEvent()

	local position
	if event=="PLAYER_LOGIN" then
		AzuarcLGSettings.Font = AzuarcLGSettings.Font or 2
		AzuarcLG.UpdateFont()
		AzuarcLG.UpdateLock()
		out("done with login")
	end
	if event=="VARIABLES_LOADED" then
		-- Set up minimap button
            out("setting up the button")
		local info = {
			icon = ALGHUD_ICON,
			position = 30, -- default only. after first use, SavedVariables used
			drag = "CIRCLE", -- default only. after first use, SavedVariables used
			left = function() AzuarcLG:Toggle() end,
			right = function() MyMinimapButton:SetEnable(ALG_MINIMAP_BUTTON,0) end,
			tooltip = "Left click: Toggle Guide Display\nRight click: Hide minimap Button\n (type '/ALG minimap on' to restore)",
			enabled = 1 -- default only. after first use, SavedVariables used
		}
		out("calling create")
	      MyMinimapButton:Create(ALG_MINIMAP_BUTTON, ALGMMBSettings, info)
		out("button created")
	end

end

-- /alg ? will show help, 
-- /alg minimap on|off will toggle the minimap button,
-- /alg # will go to a page, 
-- /alg alone toggles window
function AzuarcLG.SlashHandler(msg)

	local mmb = strfind(msg, "\?")
	if mmb then  -- help request
		DEFAULT_CHAT_FRAME:AddMessage("Usage: /alg [Pagenumber]|[Minimap ON|OFF]")
	else
		local uppermsg = strupper(msg)
		local mmb = strfind(uppermsg, SLASH_COMMAND_MMB)
		if mmb then 
			out("saw the minimap button command")
			out("mmb = " .. mmb)
			out("SLASH_COMMAND_MMB = " .. SLASH_COMMAND_MMB)
			out("tonumber(mmb) = " .. tonumber(mmb))
			out("len = " .. strlen(SLASH_COMMAND_MMB))
			local parmpos = tonumber(mmb)+strlen(SLASH_COMMAND_MMB)
			mmb = strfind(uppermsg, "ON", parmpos)
			if mmb then
				out("saw the minimap button 'ON' command")
				MyMinimapButton:SetEnable(ALG_MINIMAP_BUTTON,1)
			else 
				mmb = strfind(uppermsg, "OFF", parmpos)
				if mmb then
					out("saw the minimap button 'OFF' command")
					MyMinimapButton:SetEnable(ALG_MINIMAP_BUTTON,0)
				else
					DEFAULT_CHAT_FRAME:AddMessage("AzuarcLG: Incorrect option for minimap button - must be 'ON' or 'OFF'")

				end 
			end
		else -- not the minimap button command
 			local _,_,page = strfind(msg or "","(%d+)")
			if page then   -- found the page number 
				page = tonumber(page)
				if AzuarcLGPages[page] then
					AzuarcLG.current_page = page
					AzuarcLG.ShowPage()
				else
					DEFAULT_CHAT_FRAME:AddMessage("AzuarcLG: Page "..page.." doesn't exist.")
				end
			else -- no parms, just toggle the display
				AzuarcLG.Toggle()
			end
		end 	
	end
end

-- toggles window on/off screen
function AzuarcLG.Toggle()
	if AzuarcLGFrame:IsVisible() then
		AzuarcLGFrame:Hide()
	else
		AzuarcLGFrame:Show()
	end
end

-- window movement
function AzuarcLG.StartMoving()
	if arg1=="LeftButton" and not AzuarcLGSettings.Lock then
		AzuarcLGFrame:StartMoving()
	end
end

function AzuarcLG.StopMoving()
	if arg1=="LeftButton" then
		AzuarcLGFrame:StopMovingOrSizing()
	end
end

function AzuarcLG.Tooltip()
	local which = this:GetName()
	for i in pairs(AzuarcLG.controls) do
		if AzuarcLG.controls[i][1]==which then
			GameTooltip_SetDefaultAnchor(GameTooltip,UIParent)
			GameTooltip:AddLine(AzuarcLG.controls[i][2])
			GameTooltip:AddLine(AzuarcLG.controls[i][3],.85,.85,.85,1)
			GameTooltip:Show()
			break
		end
	end
end

-- Titlebar button clicks
function AzuarcLG.OnClick()
	local which = this:GetName()
	local new_page
	local book = AzuarcLGPages

	if which=="AzuarcLGUndo" then
		AzuarcLGEditBox:SetText(AzuarcLGPages[AzuarcLG.current_page])
	end
	if which=="ALGDDMenuButton" then
    		ToggleDropDownMenu(1, nil, ALGDropDownMenu, ALGDDMenuButton, 0, 0);
	elseif which=="AzuarcLGStart" then
		new_page = 1
	elseif which=="AzuarcLGLeft" then
		new_page = math.max(AzuarcLG.current_page-1,1)
	elseif which=="AzuarcLGRight" then
		new_page = math.min(AzuarcLG.current_page+1,table.getn(book))
	elseif which=="AzuarcLGEnd" then
		new_page = table.getn(book)
	elseif which=="AzuarcLGClose" then
		AzuarcLGFrame:Hide()
	elseif which=="AzuarcLGLock" then
		AzuarcLGSettings.Lock = not AzuarcLGSettings.Lock
		AzuarcLG.UpdateLock()
	elseif which=="AzuarcLGFont" then
		AzuarcLGSettings.Font = AzuarcLGSettings.Font+1
		AzuarcLG.UpdateFont()
	elseif which=="AzuarcLGSearch" then
		if AzuarcLGSearchFrame:IsVisible() then
			AzuarcLGSearchFrame:Hide()
		else
			AzuarcLGSearchFrame:Show()
		end
	elseif which=="AzuarcLGSearchNext" then
		AzuarcLG.DoSearch()
	end

	if new_page then
		AzuarcLG.current_page = new_page
		AzuarcLG.ShowPage()
	end
end

-- disables/enables page movement buttons depending on page
function AzuarcLG.ValidateButtons()

	-- nil for disabled, 1 for enabled
	local function set_state(button,enabled)
		if enabled then
			button:SetAlpha(1)
			button:Enable()
		else
			button:SetAlpha(.5)
			button:Disable()
		end
	end

	set_state(AzuarcLGStart,1)
	set_state(AzuarcLGLeft,1)
	set_state(AzuarcLGRight,1)
	set_state(AzuarcLGEnd,1)

	if AzuarcLG.current_page==1 then
		set_state(AzuarcLGStart)
		set_state(AzuarcLGLeft)
	end
	if AzuarcLG.current_page==table.getn(AzuarcLGPages) then
		set_state(AzuarcLGRight)
		set_state(AzuarcLGEnd)
	end
end

-- shows/updates the current page
function AzuarcLG.ShowPage()
	if not AzuarcLGFrame:IsVisible() then
		AzuarcLGFrame:Show()
	end
	if AzuarcLGPages[AzuarcLG.current_page] then
		AzuarcLGPageNum:SetText(AzuarcLG.current_page)
		AzuarcLGEditBox:SetText(AzuarcLGPages[AzuarcLG.current_page])
		AzuarcLG.ValidateButtons()
	end
end

-- refreshes window when shown
function AzuarcLG.OnShow()
	if AzuarcLG.first_use then
		-- only when the pad is first shown 
		AzuarcLG.current_page = 1
		if AzuarcLGSettings.Page then
			-- use saved page value if available 
			AzuarcLG.current_page = AzuarcLGSettings.Page 
		end 
		AzuarcLG.first_use = nil
	end
	AzuarcLG.ShowPage()
	AzuarcLGEditBox:SetWidth(AzuarcLGFrame:GetWidth()-50)
end

-- saves page when window hides
function AzuarcLG.OnHide()
	AzuarcLGPages[AzuarcLG.current_page] = AzuarcLGEditBox:GetText()
	AzuarcLGSettings.Page = AzuarcLG.current_page
	AzuarcLGUndo:Disable()
	AzuarcLGEditBox:ClearFocus()
end

-- changes border and resize grip depending on lock status
function AzuarcLG.UpdateLock()
	if AzuarcLGSettings.Lock then
		AzuarcLGFrame:SetBackdropBorderColor(0,0,0,1)
		AzuarcLGSearchFrame:SetBackdropBorderColor(0,0,0,1)
		AzuarcLGResizeGrip:Hide()
	else
		AzuarcLGFrame:SetBackdropBorderColor(1,1,1,1)
		AzuarcLGSearchFrame:SetBackdropBorderColor(1,1,1,1)
		AzuarcLGResizeGrip:Show()
	end
	AzuarcLG.MakeESCable("AzuarcLGFrame",AzuarcLGSettings.Lock)
end

-- updates EditBox font to current settings
function AzuarcLG.UpdateFont()
	if AzuarcLGSettings.Font > table.getn(AzuarcLG.fonts) then
		AzuarcLGSettings.Font = 1
	end
	local font = AzuarcLGSettings.Font
	AzuarcLGEditBox:SetFont(AzuarcLG.fonts[font][1],AzuarcLG.fonts[font][2],AzuarcLG.fonts[font][3])
end

-- adds frame to UISpecialFrames, removes frame if disable true
function AzuarcLG.MakeESCable(frame,disable)
	local idx
	for i=1,table.getn(UISpecialFrames) do
		if UISpecialFrames[i]==frame then
			idx = i
			break
		end
	end
	if idx and disable then
		table.remove(UISpecialFrames,idx)
	elseif not idx and not disable then
		table.insert(UISpecialFrames,1,frame)
	end
end
	
-- when search summoned, remove ESCability of main window (only search cleared with ESC)
function AzuarcLG.SearchOnShow()
	AzuarcLG.MakeESCable("AzuarcLGFrame","disable")
end

-- when search dismissed, restore ESCability to main window and reset search elements
function AzuarcLG.SearchOnHide()
	if not AzuarcLGSettings.Lock then
		AzuarcLG.MakeESCable("AzuarcLGFrame")
	end
	AzuarcLGSearchResults:SetText("Search:")
	AzuarcLGSearchEditBox:ClearFocus()
end

-- does a count
function AzuarcLG.SearchEditBoxOnChange()
	local search = string.lower(AzuarcLGSearchEditBox:GetText() or "")
	if string.len(search)<1 then
		AzuarcLGSearchResults:SetText("Search:")
	else
		local count = 0
		for i=1,table.getn(AzuarcLGPages) do
			count = count + (string.find(string.lower(AzuarcLGPages[i]),search,1,1) and 1 or 0)
		end
		AzuarcLGSearchResults:SetText(count.." found")
	end
end

-- performs a search for the text in the search box
function AzuarcLG.DoSearch()
	local search = string.lower(AzuarcLGSearchEditBox:GetText() or "")
	if string.len(search)<1 then
		return
	end
	local page = AzuarcLG.current_page
	for i=1,table.getn(AzuarcLGPages) do
		page = page + 1
		if page > table.getn(AzuarcLGPages) then
			page = 1
		end
		if string.find(string.lower(AzuarcLGPages[page]),search,1,1) then
			AzuarcLGPages[AzuarcLG.current_page] = AzuarcLGEditBox:GetText()
			AzuarcLG.current_page = page
			AzuarcLG.ShowPage()
			return
		end
	end
end

function AzuarcLG.OnTextChanged()
	local scrollBar = getglobal(this:GetParent():GetName().."ScrollBar")
	this:GetParent():UpdateScrollChildRect()
	local min, max = scrollBar:GetMinMaxValues()
	if ( max > 0 and (this.max ~= max) ) then
		this.max = max
		scrollBar:SetValue(max)
	end
	if this:GetText()~=AzuarcLGPages[AzuarcLG.current_page] then
		AzuarcLGUndo:Enable()
	else
		AzuarcLGUndo:Disable()
	end
end

function LoadChapter()

      local selectedItem = this.value
	out("Calling LoadChapter for " .. selectedItem)
	AzuarcLGSettings.LastChapter = selectedItem
	out("Last chapter now " .. AzuarcLGSettings.LastChapter)
	
	-- Load up this chapter
	AzuarcLGPages = ALGChapters[selectedItem] 
	AzuarcLG.current_page = 1
	AzuarcLG.ShowPage()

end

function ALGDropDownMenu_OnLoad()

	 local currentChapter = AzuarcLGSettings.LastChapter;
	 local info = UIDropDownMenu_CreateInfo(); 

       info.text       = "Act 0: Prelude";
       info.value      = 0;
       info.func       = LoadChapter;
	 info.owner = this:GetParent();
	 info.checked = (currentChapter == 0);
       UIDropDownMenu_AddButton(info);

       info.text       = "Act 1: Darkshore (12-15)";
       info.value      = 1;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 1);
       UIDropDownMenu_AddButton(info);

       info.text       = "Act 2: Bloodmyst/Darkshore (15-20)";
       info.value      = 2;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 2);
       UIDropDownMenu_AddButton(info);

       info.text       = "Act 3: Ashenvale (20-23)";
       info.value      = 3;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 3);
       UIDropDownMenu_AddButton(info);

       info.text       = "Act 4: Duskwood (23-27)";
       info.value      = 4;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 4);
       UIDropDownMenu_AddButton(info);

       info.text       = "Act 5: Ashenvale/Duskwood (27-30)";
       info.value      = 5;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 5);
       UIDropDownMenu_AddButton(info);

       info.text       = "Act 6: Thousand Needles (30-32)";
       info.value      = 6;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 6);
       UIDropDownMenu_AddButton(info);

       info.text       = "Act 7: Stranglethorn Vale (32-35)";
       info.value      = 7;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 7);
       UIDropDownMenu_AddButton(info);

       info.text       = "Act 8: Desolace (35-37)";
       info.value      = 8;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 8);
       UIDropDownMenu_AddButton(info);

       info.text       = "Act 9: Stranglethorn Vale pt. 2 (37-41)";
       info.value      = 9;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 9);
       UIDropDownMenu_AddButton(info);

       info.text       = "Act 10: TFH pt. 1 (41-43)";
       info.value      = 10;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 10);
       UIDropDownMenu_AddButton(info);

       info.text       = "Act 11: Stranglethorn Vale pt. 3 (43-46)";
       info.value      = 11;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 11);
       UIDropDownMenu_AddButton(info);

       info.text       = "Act 12: TFH pt. 2 (46-50)";
       info.value      = 12;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 12);
       UIDropDownMenu_AddButton(info);

       info.text       = "Act 13: Un'goro Crater/Plaguelands (51-52)";
       info.value      = 13;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 13);
       UIDropDownMenu_AddButton(info);

       info.text       = "Act 14: Felwood (53-54)";
       info.value      = 14;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 14);
       UIDropDownMenu_AddButton(info);

       info.text       = "Act 15: Winterspring / Plaguelands (55-56)";
       info.value      = 15;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 15);
       UIDropDownMenu_AddButton(info);

       info.text       = "Act 16: Plaguelands (57-59)";
       info.value      = 16;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 16);
       UIDropDownMenu_AddButton(info);
	
       info.text       = "Act 17: Hellfire Peninsula east (59-60)";
       info.value      = 17;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 17);
       UIDropDownMenu_AddButton(info);
	
       info.text       = "Act 18: Hellfire Peninsula west (61)";
       info.value      = 18;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 18);
       UIDropDownMenu_AddButton(info);
	
       info.text       = "Act 19: Zangarmash (62-63)";
       info.value      = 19;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 19);
       UIDropDownMenu_AddButton(info);
	
       info.text       = "Act 20: Terokkar Forest (64-65)";
       info.value      = 20;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 20);
       UIDropDownMenu_AddButton(info);
	
       info.text       = "Act 21: Nagrand (66)";
       info.value      = 21;
       info.func       = LoadChapter;
       info.owner      = this:GetParent();
	 info.checked = (currentChapter == 21);
       UIDropDownMenu_AddButton(info);
	
end

function out(text)
  -- DEFAULT_CHAT_FRAME:AddMessage(text)
  -- UIErrorsFrame:AddMessage(text, 1.0, 1.0, 0, 1, 10) 
end